//
// hp8496G attenuator driver
// OH2NLT / Juha Niinikoski 27.09.2011
//
// MCU pic16F628
// compiler htsoft PICC 8.05PL2
//

// I/O definitions
#define LCD_PORT PORTB
#define LCD_CLOCK 0x02		// LCD shift register clock
#define LCD_STR	0x01		// LCD latch strobe
#define LCD_BIT 0x04		// LCD data bit
#define DISPLAY_UPDATE 5	// LCD display update (watchdog rounds)

// switches
#define DN RB5				// SW1 step down
#define UP RB4				// SW2 Step up, Note! SW2 must be pressed during programming (PGM signal)		
#define DEBOUNCE_DELAY	50	// switch debounce delay(ms)

// relay driver Allegro A6841
#define A6841_DI  RA2		// A6841 Data In, Out 8 bit first
#define A6841_CLK RA0		// A6841 Clock, rising edge active
#define A6841_STR RA1		// A6841 Latch strobe, Active high
#define A6841_POWER RA3		// A6841 VCC

// attenuator control bits
// off = section by passed = 0dB, on = attenuator on

#define dB10_off	0x01	// hp8496G section #1 10dB
#define dB10_on		0x02

#define dB20_off	0x04	// hp8496G section #2 20dB
#define dB20_on		0x08

#define dB40_1_off	0x10	// hp8496G section #3 40dB
#define dB40_1_on	0x20

#define dB40_2_off	0x40	// hp8496G section #4 40dB
#define dB40_2_on	0x80

#define RELAY_DELAY	40		// hp8496G relay delay (ms), measured actual about 5-6ms


// included files
#include <pic.h>				// processor headers
#include "delays.c"				// simple delay function
#include "lcdriver.c"			// LCD driver


// processor config & id info

	__CONFIG(UNPROTECT & INT & WDTEN & PWRTEN & BOREN & LVPDIS);				// int osc, watchdog on


// constants
const char ver[] = {"1.0.0"};													// Version
const char id[] = {"hp8496G / OH2NLT Juha Niinikoski 27.09.2011"}; 				// id string

const char attn_table[12] = {0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110};	// attenuator steps
const char test_bit[12] = {0, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x55, 0xFF0};	// test bits 

// global variables
int attn_sel = 11;																// set max attn during power on
char disp_update = DISPLAY_UPDATE;											// display update (watchdog rounds)


// set A6841 I/O
void set_A6841(unsigned char dta)
	{
	char x;
	A6841_POWER = 1;			// set A6841 latch power on, if not yet on
	A6841_CLK = 0;				// set start condition
	A6841_STR = 0;

	for(x=0; x<8; x++)			// shift out 8 bits msb first
		{
		if(dta & 0x80)			// test MSB
			A6841_DI = 1;		// data = high
		else
			A6841_DI = 0;		// data = low

		A6841_CLK = 1;			// generate clock pulse
		dta = dta << 1;			// addjust data
		A6841_CLK = 0;
		}

	A6841_STR = 1;				// generate strobe
	asm("nop");
	A6841_STR = 0;
	A6841_DI = 0;				// leave with data = low
	}

// simple hp8496G relay setup function
// 0 = direct, 11 = max attenuation
// switch first required attenuators on then others off
// largest attenuator stage first then others
void set_attenuator(char idx)
	{
	A6841_POWER = 1;					// set A6841 latch power on
	delaym(1);

	switch(idx)
		{
		case 0:							// 0dB  direct
			set_A6841(dB40_2_off);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_off);		// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_off);		// 10dB section
			delaym(RELAY_DELAY);
		break;

		case 1:							// 10dB
			set_A6841(dB10_on);			// 10dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_2_off);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_off);		// 20dB section
			delaym(RELAY_DELAY);
		break;

		case 2:							// 20dB
			set_A6841(dB20_on);			// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_2_off);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_off);		// 10dB section
			delaym(RELAY_DELAY);
		break;

		case 3:							// 30dB
			set_A6841(dB20_on);			// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_on);			// 10dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_2_off);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);
		break;

		case 4:							// 40dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_off);		// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_off);		// 10dB section
			delaym(RELAY_DELAY);
		break;

		case 5:							// 50dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_on);			// 10dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_off);		// 20dB section
			delaym(RELAY_DELAY);
		break;

		case 6:							// 60dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_on);			// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_off);		// 10dB section
			delaym(RELAY_DELAY);
		break;

		case 7:							// 70dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_on);			// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_on);			// 10dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_off);		// first 40dB section
			delaym(RELAY_DELAY);
		break;

		case 8:							// 80dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_on);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_off);		// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_off);		// 10dB section
			delaym(RELAY_DELAY);
		break;

		case 9:							// 90dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_on);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_on);			// 10dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_off);		// 20dB section
			delaym(RELAY_DELAY);
		break;

		case 10:						// 100dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_on);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_on);			// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_off);		// 10dB section
			delaym(RELAY_DELAY);
		break;

		case 11: default:				// 110dB
			set_A6841(dB40_2_on);		// second 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB40_1_on);		// first 40dB section
			delaym(RELAY_DELAY);

			set_A6841(dB20_on);			// 20dB section
			delaym(RELAY_DELAY);

			set_A6841(dB10_on);			// 10dB section
			delaym(RELAY_DELAY);
		break;
		}

	set_A6841(0);						// all relays off
	A6841_POWER = 0;					// A6841 latch power off
	}


// Main control loop

void main(void)
	{
	asm("clrwdt");
	OPTION = 0xF0;				// no prescaler for the watchdog, about 18ms
	CMCON = 0x07;				// PORTA = Digital I/O
	TRISB = 0xF8;				// LCD lines = outputs
	TRISA = 0xF0;				// relay latch lines = outputs
	prompt_lcd();				// show start up
	xfer_lcd_buffer();			// refresh LCD

	set_attenuator(attn_sel);	// set initial value (110dB

	delaym(250);							// 0.5s delay
	delaym(250);

	put_int(attn_table[attn_sel], 0);		// display attenuator
	xfer_lcd_buffer();						// refresh LCD

	for(;;)
		{
// LCD display update
		disp_update--;
		if(disp_update == 0)
			{
			xfer_lcd_buffer();				// refresh LCD
			disp_update = DISPLAY_UPDATE;	// reload timer
			}

//		delaym(10);	// test

// check buttons
		if(UP == 0)
			{
			attn_sel++;					// increace attenuator
			if(attn_sel > 11)
				attn_sel = 11;			// limit to max

			put_int(attn_table[attn_sel], 0);		// display
			xfer_lcd_buffer();			// refresh LCD

//			set_A6841(test_bit[attn_sel]);	// test data to the I/O latch
			set_attenuator(attn_sel);	// set attenuator

//			while(UP == 0);				// wait for button release
			delaym(DEBOUNCE_DELAY);		// debounce delay
			}

		if(DN == 0)
			{
			attn_sel--;					// increace attenuator
			if(attn_sel < 0)
				attn_sel = 0;			// limit to max

			put_int(attn_table[attn_sel], 0);		// display
			xfer_lcd_buffer();			// refresh LCD

//			set_A6841(test_bit[attn_sel]);	// test data to the I/O latch
			set_attenuator(attn_sel);	// set attenuator

//			while(DN == 0);				// wait for button release
			delaym(DEBOUNCE_DELAY);		// debounce delay
			}
// sleep
		asm("sleep");					// wait for watchdog wake up
		asm("nop");
		}
	}
